from django.shortcuts import render, redirect, get_object_or_404
from django.core.paginator import Paginator
from django.http import HttpResponse
from django.template.loader import render_to_string
from django.contrib import messages
from .models import Fundi
from account.models import Account
from django.db import IntegrityError

def add_fundi(request):
    if request.method == 'POST':
        try:
            fundi_name = request.POST.get('fundi_name')
            fundi_details = request.POST.get('fundi_details')
            added_by_id = request.POST.get('added_by')
            added_by = Account.objects.get(id=added_by_id) if added_by_id else None

            new_fundi = Fundi.objects.create(
                fundi=fundi_name,
                fundi_details=fundi_details,
                added_by=added_by
            )
            messages.success(request, 'Fundi added successfully.')
            return redirect('backoffice:fundi_list')
        except IntegrityError as e:
            messages.error(request, f"Data integrity error: {str(e)}")
        except Exception as e:
            messages.error(request, f'Error adding fundi: {str(e)}')
    return render(request, "backoffice/fundi/add_fundi.html")

def fundi_list(request):
    try:
        fundi_list = Fundi.objects.all()
        paginator = Paginator(fundi_list, 10)
        page_number = request.GET.get('page')
        page_obj = paginator.get_page(page_number)

        # Export to Excel
        if request.GET.get('export') == 'true':
            response = HttpResponse(content_type='application/ms-excel')
            response['Content-Disposition'] = 'attachment; filename="fundi_list.xls"'
            context = {'page_obj': page_obj}
            html = render_to_string('backoffice/fundi/fundis_export.html', context)
            response.write(html)
            return response

        return render(request, 'backoffice/fundi/fundis.html', {'page_obj': page_obj})
    except Exception as e:
        messages.error(request, f"An error occurred: {str(e)}")
        return redirect('backoffice:fundi_list')

def edit_fundi(request, fundi_id):
    try:
        fundi = get_object_or_404(Fundi, id=fundi_id)
        if request.method == 'POST':
            fundi.fundi = request.POST.get('fundi_name')
            fundi.fundi_details = request.POST.get('fundi_details')
            fundi.save()
            messages.success(request, 'Fundi updated successfully.')
            return redirect('backoffice:fundi_list')
        return render(request, 'backoffice/fundi/edit_fundi.html', {'fundi': fundi})
    except Exception as e:
        messages.error(request, f"An error occurred: {str(e)}")
        return redirect('backoffice:fundi_list')

def delete_fundi(request, fundi_id):
    try:
        fundi = get_object_or_404(Fundi, id=fundi_id)
        fundi_name = fundi.fundi
        fundi.delete()
        messages.success(request, f"Fundi '{fundi_name}' has been deleted successfully.")
    except Exception as e:
        messages.error(request, f"An error occurred while deleting the fundi: {str(e)}")
    return redirect('backoffice:fundi_list')